/*
 * *********************************************************************************************************************
 *
 *          !!
 *        .F88X
 *        X8888Y
 *      .}888888N;
 *        i888888N;        .:!              .I$WI:
 *          R888888I      .'N88~            i8}+8Y&8"l8i$8>8W~'>W8}8]KW+8IIN"8&
 *          .R888888I    .;N8888~          .X8'  "8I.!,/8"  !%NY8`"8I8~~8>,88I
 *            +888888N;  .8888888Y                                  "&&8Y.}8,
 *            ./888888N;  .R888888Y        .'}~    .>}'.`+>  i}!    "i'  +/'  .'i~  !11,.:">,  .~]!  .i}i
 *              ~888888%:  .I888888l      .]88~`1/iY88Ii+1'.R$8$8]"888888888>  Y8$  W8E  X8E  W8888'188Il}Y88$*
 *              18888888    E8888881    .]W%8$`R8X'&8%++N8i,8N%N8+l8%`  .}8N:.R$RE%N88N%N$K$R  188,FE$8%~Y88I
 *            .E888888I  .i8888888'      .:$8I;88+`E8R:/8N,.>881.`$8E/1/]N8X.Y8N`"KF&&FK!'88*."88K./$88%RN888+~
 *            8888888I  .,N888888~        ~88i"8W,!N8*.I88.}888%F,i$88"F88"  888:E8X.>88!i88>`888*.}Fl1]*}1YKi'
 *          i888888N'      I888Y          ]88;/EX*IFKFK88X  K8R  .l8W  88Y  ~88}'88E&%8W.X8N``]88!.$8K  .:W8I
 *        .i888888N;        I8Y          .&8$  .X88!  i881.:%888>I88  ;88]  +88+.';;;;:.Y88X  18N.,88l  .+88/
 *      .:R888888I
 *      .&888888I                                          Copyright (c) 2016-2020.  博锐尚格科技股份有限公司
 *        ~8888'
 *        .!88~                                                                     All rights reserved.
 *
 * *********************************************************************************************************************
 */

import { SGraphItem, SAnchorItem } from "@persagy-web/graph/lib";
import { SPoint, SPainter, SColor } from "@persagy-web/draw";
import { Example } from "../index";
import { SBaseIconTextEdit, SBaseTextEdit } from "@persagy-web/edit";
import { svgTobase64, Marker, ItemOrder } from "@persagy-web/big/lib";
import { v1 as uuidv1 } from "uuid";

/**
 * 编辑公共设备类(mark 类)
 *
 * @author 韩耀龙 <han_yao_long@163.com>
 */
export class SPubEquipment extends SBaseIconTextEdit {

    /** 图例url */
    private _url: string = "";
    set url(val) {
        this._url = val;
        this.initUrl();
    } // set url()
    get url(): string {
        return this._url;
    } // get url()


    //  设备附加数据
    anotherMsg: string = "";

    /**
     * 构造函数
     *
     * @param parent    指向父对象
     * @param data      数据
     */
    constructor(parent: SGraphItem | null, data: Marker) {
        super(parent);
        this.zOrder = 9700;
        this.isTransform = false;
        this.sWidth = 32;
        this.sHeight = 32;
        this.img.showSelect = false;
        this.moveable = true;
        this.selectable = true;
        this.data = data;
    } // Constructor

    /**
     * 设置 data 时对 item 做设置
     */
    initData() {
        // this.data 不能为空
        if (!this.data) return;

        // 判断 size 是否存在
        if (this.data.size) {
            this.sWidth = this.data.size.width;
            this.sHeight = this.data.size.height;
        }

        this.img.connect("onMove", this, this.changeAnchorPoint.bind(this));


        // 判断是否有锚点数据
        const anchorPoint = [
            { pos: { x: this.img.x, y: this.img.y }, anchorId: uuidv1() }
        ];

        // 没锚点则默认中心点为锚点
        this.anchorList = anchorPoint.map(t => {
            let item = new SAnchorItem(this);
            // 是否存在 anchorId
            if (t.anchorId) {
                item.id = t.anchorId;
            }

            item.moveTo(t.pos.x, t.pos.y);
            return item;
        });
        this.zOrder = ItemOrder.RelationOrder + 10;


        this.showAnchor = false;
        this.anotherMsg = this.data.properties.anotherMsg
            ? this.data.properties.anotherMsg
            : "";
        this.x = this.data.pos.x;
        this.y = this.data.pos.y;
        this.moveable = true;
        this.selectable = true;
    }

    initUrl() {
        svgTobase64(this.url)
            .then((res) => {
                this.img.url = res ? res : "";
            })
            .catch((res) => {
                this.img.url = res;
            });
    }

    /**
     * 文本位置移动回调函数
     * @param textItem 移动的文本元素
     * @param pointList 移动前后位置坐标信息
     */
    textMove(textItem: SBaseTextEdit, pointList: SPoint[]) {
        this.moveText();
    }
    moveText() { }

    /**
     * 返回对象储存的相关数据
     *
     * @return	对象储存的相关数据
     */
    toData(): any {
        if (this.data) {
            // 数据中存在大小 size 属性
            if (this.data.size) {
                this.data.size.width = this.sWidth;
                this.data.size.height = this.sHeight;
            }
            this.data.style.default.text = JSON.stringify(this.textItemList);
            this.data.pos.x = this.pos.x;
            this.data.pos.y = this.pos.y;
            this.data.style.default.zorder = this.zOrder;
        }
        return this.data
    }

    /**
     * Item 绘制操作
     *
     * @param painter    绘制对象
     */
    onDraw(painter: SPainter): void {
        super.onDraw(painter);
    } // Function onDraw()
}
